# -*- coding: UTF-8 -*-

# ereseva.py
#
#  Copyright (c) 2007 INdT (Instituto Nokia de Technologia)
#                2007 Kenneth Rohde Christiansen
#
#  Author: Kenneth Rohde Christiansen <kenneth.christiansen@gmail.com>
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA

import pygtk
pygtk.require("2.0")
import gtk
import tapioca
import emoticons

from debug import debug
from tapioca_thread import TapiocaThread

from chat_window import ChatWindow
from local_contact_store import LocalContactStore
from application_window import ApplicationWindow
from chat_window import ChatWindow
from videochat_window import VideoChatWindow
from dialogs import AboutDialog
from dialogs import ConnectionProblemDialog
from dialogs import IncomingCallDialog
from dialogs import OutgoingCallDialog
from dialogs import SignOutDialog
from emoticon_dialog import EmoticonDialog

import icon_theme
import util

from gettext import gettext as _
from gettext import bindtextdomain, textdomain

from os import sep
from locale import setlocale, LC_ALL

import dbus
import dbus.service
import dbus.glib
from dbus_service import Service

LOCALE_PREFIX = "%susr" % (sep)
LOCALE_DIR = "%s%sshare%slocale" % (LOCALE_PREFIX, sep, sep)

print LOCALE_DIR

## variables ##

app_name = "ereseva"
app_version = "0.0.3"

gtk.gdk.threads_init()
tapioca.thread_init(False)

bindtextdomain(app_name, LOCALE_DIR)
textdomain(app_name)

CONFIG_PATH = "/apps/ereseva"


class MenuBar(gtk.UIManager):
    """
    A menu bar using the UIManager.
    """

    def __init__(self, parent):
        """
        A constructor for the menu bar.
        """
        self.parent = parent

        actions = [
            #( "About", gtk.STOCK_ABOUT, "_About", None, None, self.on_about_menuitem_clicked),
            ( "Showcontactlist", gtk.STOCK_INDEX, _("_Show Contact List"), None, None, self.on_showcontactlist_menuitem_clicked),
            ( "StatusMenu", None, _("_Status")),
            #( "Available", 'status-online', _("_Available"), None, None, self.on_signout_menuitem_clicked),
            #( "Busy", 'status-busy', _("_Busy"), None, None, self.on_signout_menuitem_clicked),
            #( "Away", 'status-away', _("A_way"), None, None, self.on_signout_menuitem_clicked),
            ( "Signout", gtk.STOCK_CLOSE, _("_Sign Out"), None, None, self.on_signout_menuitem_clicked),
            ( "Quit", gtk.STOCK_QUIT, _("_Quit..."), None, None, self.on_quit_menuitem_clicked),
        ]

        actiongroup = gtk.ActionGroup("PopupActions")
        actiongroup.add_actions(actions)

        gtk.UIManager.__init__(self)

        self.insert_action_group(actiongroup, -1)

        self.add_ui_from_string( \
        '<ui>' \
        '  <popup action="popup" name="popup">' \
        '    <menuitem action="Showcontactlist" name="Showcontactlist"/>' \
        '    <separator name="sep1"/>' \
        #'    <menu action="StatusMenu" name ="StatusMenu">'\
        #'      <menuitem action="Available" name="Available"/>' \
        #'      <menuitem action="Busy" name="Busy"/>' \
        #'      <menuitem action="Away" name="Away"/>' \
        '      <menuitem action="Signout" name="Signoff"/>' \
        #'    </menu>'\
        '    <separator name="sep2"/>' \
        '    <menuitem action="Quit" name="Quit"/>' \
        '  </popup>' \
        '</ui>')


    def new(self):
        self.ensure_update()


    def on_about_menuitem_clicked(self, widget, *args):
        """
        This function pops up an about dialog.

        """
        dialog = AboutDialog()
        dialog.run()
        dialog.destroy()


    def on_showcontactlist_menuitem_clicked(self, widget, *args):
        """
        This callback function show the contact list.
        """

        print widget
        print dir(widget)
        self.parent.application_window.present()


    def on_signout_menuitem_clicked(self, widget, *args):
        """
        This callback function just set the presence status to offline.
        """
        if len(self.parent.chat_windows) > 0:
            dialog = SignOutDialog()
            response = dialog.run()
            dialog.destroy()

            if response != gtk.RESPONSE_YES:
                return

        self.parent.sign_out(self, None)

    def on_quit_menuitem_clicked(self, widget, *args):
        """
        This callback function closes the application by quitting
        the GTK+ main loop.
        """
        self.parent.quit()


class Ereseva(object):

    connection = None
    contact_list = None
    open_text_channel = {}
    open_stream_channel ={}
    text_channel_handlers = {}
    stream_channel_handlers = {}
    chat_windows = {}
    client_created_text_channels = set()
    client_created_stream_channels = set()
    contacts_contacted_in_session = set()
    current_connections = {}
    account = ""
    last_messages = {}
    requested_disconnect = False

    def __init__(self):
        """
        Inits the ereseva application, creating the tray and the main
        application, as well as loads the emoticons.
        """

        self.tapiocathread = TapiocaThread()
        self.tapiocathread.start()

        session_bus = dbus.SessionBus()
        bus_name = dbus.service.BusName('org.tapioca.Ereseva', bus=session_bus)
        self.service = Service(bus_name, self)

        self.factory = tapioca.ManagerFactory()

        self.local_contact_store = LocalContactStore()

        self.application_window = ApplicationWindow(self,\
                self.local_contact_store)
        self.application_window.set_property("icon-name", 'ereseva')

        logo_pixbuf = icon_theme.icon_theme_get_default().\
                load_icon("ereseva", 32, 0)
        self.trayicon = gtk.status_icon_new_from_pixbuf(\
                    self.local_contact_store.\
                    get_presence_icon_from_presence(\
                    tapioca.PRESENCE_OFFLINE)\
                )
        self.trayicon.set_tooltip(_("Offline - Ereseva"))

        self.trayicon.connect('popup_menu', self.trayaction_menu)
        self.trayicon.connect('activate', self.trayaction_activate)

        self.menu = MenuBar(self).get_widget("/popup")

        self.emoticons = emoticons.Emoticons()

        # Ugly workaround
        self.last_message_received = None


    def trayaction_activate(self, status_icon):
        """
        Hide or show the main application window (depending on
        the current state) when clicking on the tray icon.
        """

        if self.application_window.get_property('visible'):
            self.application_window.hide()
        else:
            self.application_window.present()


    def trayaction_menu(self, status_icon, button, activate_time):
        self.menu.popup(None, None, gtk.status_icon_position_menu, \
                button, activate_time, status_icon)


    def main(self):
        """
        The main function of ereseva.
        """

        gtk.main()


    def quit(self):
        """
        The quit funcion of ereseva. If a connection is in use,
        the function first signs out and closes the connection.
        """

        self.application_window.save_geometry_and_position_state()

        if self.connection != None:
            self.sign_out(None, None)
            self.connection.disconnect()

        self.tapiocathread.quit()

        # False = Do not wait until threads are done
        tapioca.thread_shutdown(False)

        gtk.main_quit()


    def dispose_connection_resources(self):
        """
        Function that disposes all resources associated with
        a connnection, so that a new connection can be made.
        """

        for contact in self.local_contact_store.get_all_contacts():
            del contact # dispose and disconnect signals

        contact_list = self.local_contact_store.get_contact_list()
        del contact_list


    #def set_default_parameters(self, parameters):
    #    parameters.set_value_as_string("resource", "Ereseva.v%s" % app_version)

    #    return parameters


    def get_gtalk_jabber_profile(self, manager, account, password):
        profile = manager.get_profile("jabber")

        self.account = account
        server_name = util.get_server_name(account)

        parameters = profile.get_all_parameters()

        for param in parameters:
            pname = param.get_name()

            if pname == 'account':
                param.set_value_as_string(account)
            elif pname == 'password':
                param.set_value_as_string(password)
            elif pname == 'resource':
                param.set_value_as_string('Ereséva')
            elif pname == 'server':
                param.set_value_as_string(server_name)
            elif pname == 'port':
                param.set_value_as_uint(5223)
            elif pname == 'old-ssl':
                param.set_value_as_boolean(True)
            elif pname == 'ignore-ssl-errors':
                param.set_value_as_boolean(True)

        return profile


    def login(self, account=None, password=None):
        self.requested_disconnect = False

        def do_connection_problem_dialog():
            dialog = ConnectionProblemDialog()
            response = dialog.run()
            dialog.destroy()


            if response == gtk.RESPONSE_YES:
                self.tapiocathread.idle_add(self.login, \
                        username, password)
            else:
                self.application_window.set_disconnected_view()

            return

        if not self.factory:
            do_connection_problem_dialog()

        manager = self.factory.get_manager("jabber")

        if not manager:
            do_connection_problem_dialog()

        profile = self.get_gtalk_jabber_profile(manager, account, password)
        self.connection = manager.request_connection(profile)

        if not self.connection:
            do_connection_problem_dialog()

        self.connection.connect("status-changed", self.on_status_changed)
        self.connection.connect("channel-created", self.on_channel_created)

        self.connection.make_connection()


    def connect_account (self, widget, username, password):
        self.tapiocathread.idle_add(self.login, username, password)


    def load_contact_list (self):
        contacts = self.contact_list.get_known()
        for c in contacts:
            s_status = c.get_subscription_status()
            a_status = c.get_authorization_status()
            blocked = c.is_blocked()
            #print '%s --- %s -- %s' % (c.get_uri(), s_status == \
            #        tapioca.\
            #        SUBSCRIPTION_STATUS_REMOTE_PENDING,\
            #        a_status == tapioca.\
            #        AUTHORIZATION_STATUS_LOCAL_PENDING)
            #print 'is_blocked: %s' % blocked
        self.application_window.add_contacts(contacts)
        self.application_window.update_pending_contacts()


    def sign_out(self, widget, data):
        """
        Function to sign out of the current connection. If chat window
        are opened, they will automatically be closed. Presence will
        be changed to Offline before the actual server disconnect.
        """

        windows = []

        # Don't work on the self.chat_windows directly as
        # it is modified dynamically by the destroy method.
        for key in self.chat_windows:
            windows.append(self.chat_windows[key].get_widget())

        for window in windows:
            window.destroy()

        self.requested_disconnect = True

        self.application_window.set_disconnected_view()
        self.set_presence (tapioca.PRESENCE_OFFLINE, None)
        self.connection.disconnect()


    def set_presence(self, presence=None, custom_message=None):
        """
        Setter function to set the presence of the current
        user. It also updated the tray icon and the associated
        tooltip.
        """

        user = self.connection.get_user_contact()
        contact_id = user.get_uri()

        if not presence:
            presence =self.local_contact_store.get_presence(contact_id)

        if custom_message == None:
            custom_message = ''
            generic_message = self.local_contact_store.\
                    get_presence_message_from_presence(\
                    presence)
            self.trayicon.set_tooltip(generic_message +\
                    ' - Ereséva')
        else:
            self.trayicon.set_tooltip(custom_message +\
                    ' - Ereséva')

        pixbuf = self.local_contact_store.\
                get_presence_icon_from_presence(presence)
        self.trayicon.set_from_pixbuf(pixbuf)

        self.local_contact_store.set_presence(contact_id, presence)
        self.local_contact_store.set_presence_message(contact_id,\
                custom_message)
        user.set_presence_with_message(presence, custom_message)


    def set_alias(self, custom_alias):
        """
        Setter function to set the alias of the current user
        """

        user = self.connection.get_user_contact()

        if custom_alias == None:
            custom_message = ''

        user.set_alias(custom_alias)


    def on_status_changed(self, conn, status, reason):
        if conn != self.connection:
            return

        if status == tapioca.CONNECTION_STATUS_CONNECTED:
            user = conn.get_user_contact()
            alias = user.get_alias()
            my_id = user.get_uri()

            my_protocol = conn.get_protocol()

            #FIXME:
            #try:
                # Already connected to this account. CM bug?
                #if my_protocol == self.current_connections[my_id]:
                    #return
            #except KeyError, e:
                #pass

            self.current_connections[my_id] = my_protocol

            conn.get_open_channels()
            self.contact_list = conn.get_contactlist()
            self.local_contact_store.set_contact_list(self.contact_list)
            self.local_contact_store.set_contact(my_id, user)

            self.contact_list.connect('authorization-requested', \
                    self.on_authorization_requested)

            self.tapiocathread.idle_add(self.load_contact_list)

            self.application_window.set_id(my_id)
            self.application_window.set_connected_view()

            self.application_window.set_user_alias(user.get_alias())
            user.connect("avatar-received", self.on_user_avatar_received)
            user.request_avatar()

            user.set_capabilities(tapioca.CAPABILITY_TEXT & \
                    tapioca.CAPABILITY_AUDIO)# & \
                    #tapioca.CAPABILITY_VIDEO)

            # FIXME: there is some problem with the binding for the
            # TpaCapabiblity enum, so we need this line.
            user.set_capabilities(3)

        elif status == tapioca.CONNECTION_STATUS_DISCONNECTED:
            self.dispose_connection_resources()

            self.local_contact_store.clear()
            self.application_window.contact_list_widget.clear()

            self.tapiocathread.clear_tasks()

            if not self.requested_disconnect:
                dialog = ConnectionProblemDialog()
                response = dialog.run()
                dialog.destroy()

                if response == gtk.RESPONSE_YES:
                    pass
                    #self.tapiocathread.idle_add(self.login,\
                    #                            username, password)
                else:
                    pass
                    self.application_window.set_disconnected_view()


    def on_change_presence(self, widget, presence, custom_message=None):
        self.tapiocathread.idle_add(self.set_presence, presence, \
                                    custom_message)


    def on_user_avatar_received(self, user_contact, avatar):
        self.application_window.set_user_avatar(avatar)


    def start_chat_session(self, contact):
        """
        Method to request a chat session.
        If the channel session is already open. no new session
        is created.
        """

        contact_id = contact.get_uri()

        if not self.open_text_channel.has_key(contact_id):
            self.client_created_text_channels.add(contact_id)
            self.connection.create_channel(\
                    tapioca.CHANNEL_TYPE_TEXT, contact)


    def start_voip_session(self, contact):
        """
        Method to request a voip session.
        If the channel session is already open. no new session
        is created.
        """

        print '[start_voip_session]'

        contact_id = contact.get_uri()

        if not self.open_stream_channel.has_key(contact_id):
            self.client_created_stream_channels.add(contact_id)
            self.connection.create_channel(\
                    tapioca.CHANNEL_TYPE_STREAM, contact)

        #if not self.open_text_channel.has_key(contact_id):
        #    self.start_chat_session(contact)


    def stop_voip_session(self, contact):
        """
        Method to close a voip session.
        """

        print '[start_voip_session]'

        contact_id = contact.get_uri()

        if self.open_stream_channel.has_key(contact_id):
            stream_channel = self.open_stream_channel[contact_id]
            for stream in stream_channel.get_streams():
                stream_channel.remove_stream(stream)
            stream_channel.close()


    def accept_voip_session(self, contact):
        """
        Method to accept a voip session.
        """

        print '[accept_voip_session]'

        contact_id = contact.get_uri()

        if self.open_stream_channel.has_key(contact_id):
            stream_channel = self.open_stream_channel[contact_id]
            print 'stream_channel.has_joined: ', stream_channel.has_joined()
            stream_channel.join()


    def start_webcam_session(self, widget, contact):
        """
        Method to request a webcam session.
        If the channel session is already open. no new session
        is created.
        """

        videochat = VideoChatWindow(contact.get_uri(), \
                self.local_contact_store)


    def stop_webcam_session(self, contact):
        """
        Method to close a webcam session.
        """
        pass


    def on_authorization_requested(self, contact_list, contact):
        contact_id = contact.get_uri()
        self.local_contact_store.set_contact(contact_id, contact)
        self.application_window.update_pending_contacts(contact_id)


    def _end_chat(self, widget, channels, last_messages):

        text_channel = channels[ChatWindow.CHANNEL_TEXT]
        stream_channel = channels[ChatWindow.CHANNEL_STREAM]

        if text_channel:
            contact_id = text_channel.get_target().get_uri()
        else:
            contact_id = stream_channel.get_target().get_uri()

        self.last_messages[contact_id] = last_messages

        if self.text_channel_handlers.has_key(text_channel):
            for handler in self.text_channel_handlers[text_channel]:
                text_channel.disconnect(handler)

            del self.text_channel_handlers[text_channel]

        if self.stream_channel_handlers.has_key(stream_channel):
            for handler in self.stream_channel_handlers[stream_channel]:
                stream_channel.disconnect(handler)

            del self.stream_channel_handlers[stream_channel]

        if self.chat_windows.has_key(text_channel):
            del self.chat_windows[text_channel]

        if self.chat_windows.has_key(stream_channel):
            del self.chat_windows[stream_channel]

        if self.open_text_channel.has_key(contact_id):
            del self.open_text_channel[contact_id]

        if self.open_stream_channel.has_key(contact_id):
            del self.open_stream_channel[contact_id]

        if text_channel:
            text_channel.close()
            del text_channel

        if stream_channel:
            stream_channel.close()
            del stream_channel


    def on_stream_state_changed(self, stream, state):
        """
        Callback function for when a stream changes state.
        """

        print '[on_stream_state_changed] -----------------------------'
        print stream
        print state

        if state == tapioca.STREAM_STATE_PLAYING:
            stream.mute_in(False)
            stream.mute_out(False)
            stream.set_volume(90)

            channel = stream.get_channel()
            chat_window = self.get_chat_window_for_channel(channel)
            chat_window.update_view(ChatWindow.VIEW_TALKING)
        print '-------------------------------------------------------'

        '''
        if stream.get_stream_type() == tapioca.STREAM_TYPE_VIDEO and \
            state == tapioca.STREAM_STATE_PLAYING:
            print "Setting window"

            #stream.set_window_id(self.window.window.xid)
            #stream.add_preview_window_id(self.preview.window.xid)
        '''


    def on_stream_added(self, channel, stream):
        """
        Callback function for when a stream is created.
        """

        print '[on_stream_added] -------------------------------------'
        print stream.get_state()
        print '-------------------------------------------------------'

        stream.connect('state-changed', self.on_stream_state_changed)

        chat_window = self.chat_windows[channel]
        print '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'
        print chat_window
        chat_window.add_channel(channel)

        stream_type = stream.get_stream_type()
        if stream_type == tapioca.STREAM_TYPE_AUDIO:
            print "New audio stream added!"
        elif stream_type == tapioca.STREAM_TYPE_VIDEO:
            print "New video stream added!"


    def on_channel_created(self, conn, channel):
        """
        Callback function for when a new channel (chat, video, etc)
        has been created.
        """

        if channel.get_channel_type() == tapioca.CHANNEL_TYPE_STREAM:
            self.on_stream_channel_created(conn, channel)
        elif channel.get_channel_type() == tapioca.CHANNEL_TYPE_TEXT:
            self.on_text_channel_created(conn, channel)
        else:
            channel.close()

        return


    def on_stream_channel_created(self, conn, channel):
        """
        Callback function for when a new stream channel has
        been created.
        """

        print '[on_stream_channel_created] ---------------------------'
        print channel
        print '-------------------------------------------------------'

        contact = channel.get_target()
        contact_id = contact.get_uri()

        client_created_channel = contact_id in \
                self.client_created_stream_channels

        self.open_stream_channel[contact_id] = channel
        self.contacts_contacted_in_session.add(channel)

        if not self.open_text_channel.has_key(contact_id):
            self.start_chat_session(contact)

        self.stream_channel_handlers[channel] = (
            channel.connect("stream-added", self.on_stream_added),
            channel.connect("closed", self.on_stream_channel_closed)
        )

        chat_window = self.get_chat_window_for_channel(channel)
        chat_window.set_urgency_hint(True)

        if client_created_channel:
            print 'client_created_channel'
            self.client_created_stream_channels.remove(contact_id)
            types = [tapioca.STREAM_TYPE_AUDIO]
            channel.request_streams(channel.get_target(), types)
            chat_window.update_view(ChatWindow.VIEW_CALLING)
        else:
            print 'someone wants to start a voip session...'
            chat_window.update_view(ChatWindow.VIEW_RECEIVING_CALL)


        '''
        contact = channel.get_target()
        contact_id = contact.get_uri()
        contact_alias = self.local_contact_store.get_alias(contact_id)

        def do_incoming_call(widget, response):
            widget.destroy()
            print 'Incoming Call: ',
            if response == IncomingCallDialog.RESPONSE_IGNORE:
                print 'ignore'
                print channel
                streams = channel.get_streams()
                print streams

                try:
                    stream = streams[0]
                    print stream
                    print stream.get_state()
                    print stream.get_stream_type()
                except Exception, e:
                    print e
                #channel.remove_stream(stream)
            else:
                print 'answer'
                # FIXME: check this
                #types = [tapioca.STREAM_TYPE_AUDIO, tapioca.STREAM_TYPE_VIDEO]
                types = [tapioca.STREAM_TYPE_AUDIO]
                channel.request_streams(channel.get_target(), types)

                if not channel.has_joined():
                    channel.join()

        dialog = IncomingCallDialog(contact_alias)
        dialog.connect('user-response', do_incoming_call)
        '''


    def on_text_channel_created(self, conn, channel):
        """
        Callback function for when a new text channel has been
        created.
        """

        contact_id = channel.get_target().get_uri()

        pending_messages = channel.get_pending(True)

        client_created_channel = contact_id in \
                self.client_created_text_channels

        if client_created_channel:
            self.client_created_text_channels.remove(contact_id)

        elif not pending_messages:
            channel.close()
            return

        self.open_text_channel[contact_id] = channel
        self.contacts_contacted_in_session.add(channel)

        chat_window = self.get_chat_window_for_channel(channel)
        chat_window.set_urgency_hint(True)

        #chat_window.update_view()

        if not chat_window.get_text_channel():
            chat_window.add_channel(channel)

        #if not client_created_channel and \
        #    channel not in self.contacts_contacted_in_session:
        if not client_created_channel:
            for msg in pending_messages:
                chat_window.insert_message(msg.get_contents())
                self.last_message_received = msg

        self.text_channel_handlers[channel] = (
            channel.connect('message-received', self.on_message_received),
            channel.connect('message-sent', self.on_message_sent),
            channel.connect('closed', self.on_channel_closed)
        )


    def on_channel_closed(self, channel):
        """
        Callback function for when a channel is closed.
        """

        print '[on_channel_closed], ',
        print channel
        print type(channel)


    def on_stream_channel_closed(self, channel):
        """
        Callback function for when a stream channel is closed.
        """

        print '[on_stream_channel_closed] ----------------------------'
        print channel
        print '-------------------------------------------------------'

        contact_id = channel.get_target().get_uri()

        if self.open_stream_channel.has_key(contact_id):
            chat_window = self.chat_windows[channel]
            chat_window.update_view()
            del self.chat_windows[channel]
            del self.open_stream_channel[contact_id]

        chat_window.update_view()


    def on_message_received(self, channel, message):
        """
        Callback function for when a message has been received.
        The function insert sthe message in the associated 
        chat window.
        """

        debug(0, "message_received: %s" % str(channel))

        if self.last_message_received and \
                self.last_message_received.get_contents() == \
                message.get_contents():
            self.last_message_received = None
            return

        chat_window = self.get_chat_window_for_channel(channel)
        contents = message.get_contents()
        chat_window.insert_message(contents)


    def on_message_sent(self, channel, message):
        """
        Callback function for when a message has been sent.
        The function inserts the message in the associated
        chat window.
        """

        chat_window = self.get_chat_window_for_channel(channel)
        contents = message.get_contents()
        chat_window.insert_message(contents, True)


    def get_chat_window_for_channel(self, channel):
        """
        Function that returns a chat window for a given
        channel. If a window does not already exist, one
        will be created. If there has been chatted with
        the contact within the current connection session,
        the last messages are added to the new chat window.
        """

        contact = channel.get_target()
        contact_id = contact.get_uri()
        text_channel = None
        stream_channel = None
        chat_window = None

        if type(channel) == tapioca.TextChannel:
            text_channel = channel
            self.open_text_channel[contact_id] = text_channel

            if self.open_stream_channel.has_key(contact_id):
                stream_channel = self.open_stream_channel[contact_id]
                chat_window = self.chat_windows[stream_channel]
            elif self.chat_windows.has_key(text_channel):
                    chat_window = self.chat_windows[text_channel]

        elif type(channel) == tapioca.StreamChannel:
            stream_channel = channel
            self.open_stream_channel[contact_id] = stream_channel

            if self.open_text_channel.has_key(contact_id):
                text_channel = self.open_text_channel[contact_id]
                chat_window = self.chat_windows[text_channel]
            elif self.chat_windows.has_key(stream_channel):
                    chat_window = self.chat_windows[stream_channel]

        if not chat_window:
            if text_channel and self.last_messages.has_key(contact_id):
                last_messages = self.last_messages[contact_id]
            else:
                last_messages = []

            chat_window = ChatWindow(self.tapiocathread, channel, \
                self.local_contact_store, self, last_messages)
            capabilities = contact.get_capabilities()
            chat_window.set_capabilities(capabilities)
            chat_window.connect('chat-window-closed', self._end_chat)

        if text_channel:
            self.chat_windows[text_channel] = chat_window
        if stream_channel:
            self.chat_windows[stream_channel] = chat_window

        return chat_window


def main():
    """
    Run the Ereséva main application.
    """

    debug.level = 2

    ereseva = Ereseva()
    ereseva.main()

if __name__ == "__main__":
    main()

