# -*- coding: UTF-8 -*-

#i tapioca-thread.py 
#
#  Copyright (c) 2007 INdT (Instituto Nokia de Technologia)
#                2007 Kenneth Rohde Christiansen
#
#  Author: Kenneth Rohde Christiansen <kenneth.christiansen@gmail.com>
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA

import threading

import Queue
import debug

class TapiocaThread(object):
    def __init__(self):
        """
        Constructor.
        """

        self.run = True
        self.queue = Queue.Queue(-1)
        self.thread = threading.Thread(target=self._loop, name="TapiocaThread")

    def start(self):
        """
        Start the thread.
        """

        self.thread.start()

    def idle_add(self, function, *args, **kargs):
        """
        Function to add a function to the threads main loop
        """

        self.queue.put((function, args, kargs))

    def _loop(self):
        """
        The threads main loop
        """

        while self.run:
            function, args, kargs = self.queue.get()
            try:
                function(*args, **kargs)
            except Exception, e:
                debug(0, "Function threw exception: %s [%s], args=%s, kargs=%s\n%s" \
                    % (function.func_name, function, args, kargs, e))

    def clear_tasks(self):
        """
        Clear the thread queue.
        """

        while self.queue.empty == False:
            self.queue.get()

    def quit(self):
        """
        Shutdown and quit the thread.
        """

        self.clear_tasks()

        def quit_thread():
            self.run = False

        self.idle_add(quit_thread)
